# Copyright 2019 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from multiprocessing import Queue

############Experimental Code###################


global global_cl_inst


def setClAgentInst(clInst):
    global global_cl_inst
    global_cl_inst = clInst


def getClAgentInst():
    global global_cl_inst
    return global_cl_inst

###############################################


def create_message_queue(*args, **kwargs):

    logger = kwargs.get('logger')
    conf = kwargs.get('conf')
    return (NetQPriorityQueue.create_priority_queue(conf, logger))

class QueueElem(object):

    def __init__(self, priority, message_type, data):
        self.priority = priority
        self.message_type = message_type
        self.data = data


class NetQPriorityQueueWrapper(object):

    'NetQ Wrapper Priority Queue implementation'

    def __init__(self, maxsize=1000, conf=None, max_num_queue=5, logger=None):
        self.maxsize = maxsize
        self._logger = logger
        self._conf = conf
        self._max_num_queue = max_num_queue
        self.pq = []
        for priority in range(0, max_num_queue):
            priority_queue = Queue(maxsize)
            priority_queue.cancel_join_thread()
            self.pq.append(priority_queue)


    @property
    def max_num_queue(self):
        return self._max_num_queue

    def is_full(self):
        for queue_id in range(0, self._max_num_queue):
            if (self.pq[queue_id].qsize() >= self.maxsize):
                return True
        return False

    def get_reader_fds(self):
        fd_set = []
        for queue_id in range(0, self._max_num_queue):
            try:
                if self.pq[queue_id] is not None:
                    fd_set.append(self.pq[queue_id]._reader)
                else:
                    self._logger.warn("Queue Destroyed {}".format(queue_id))
            except Exception as e:
                self._logger.error("Exception Caught Idx {} {}".format(queue_id, e))
        return fd_set

    def len(self, priority_idx):
        return self.pq[priority_idx].qsize()

    def putq(self, elem):
        try:
            self.pq[elem.priority].put_nowait((elem.priority, elem.message_type, elem))
        except Exception:
            self._logger.debug("Queue full length:{}".format(self.len(elem.priority)))
            
    def get_buffer(self, priority_idx):
        while not self.pq[priority_idx].empty():
            yield self.pq[priority_idx].get()


class NetQPriorityQueue(object):

    PQ = None

    @classmethod
    def create_priority_queue(cls, conf, logger):
        try:
            cls.PQ = NetQPriorityQueueWrapper(maxsize=10000, conf=conf, logger=logger)
        except Exception as e:
            logger.error("Exception {}".format(e))
            cls.PQ = None
        return cls.PQ




